@@ -1,25 +1,15 @@
-/* covar.f -- translated by f2c (version 20050501).
-   You must link the resulting object file with libf2c:
-	on Microsoft Windows system, link with libf2c.lib;
-	on Linux or Unix systems, link with .../path/to/libf2c.a -lm
-	or, if you install libf2c.a in a standard place, with -lf2c -lm
-	-- in that order, at the end of the command line, as in
-		cc *.o -lf2c -lm
-	Source for libf2c is in /netlib/f2c/libf2c.zip, e.g.,
-
-		http://www.netlib.org/f2c/libf2c.zip
-*/
-
-#include <math.h>
 #include "cminpack.h"
+#include <math.h>
+#define real __cminpack_real__
 
-/* Subroutine */ void covar(int n, double *r, int ldr, 
-	const int *ipvt, double tol, double *wa)
+__cminpack_attr__
+void __cminpack_func__(covar)(int n, real *r, int ldr, 
+	const int *ipvt, real tol, real *wa)
 {
     /* Local variables */
     int i, j, k, l, ii, jj;
     int sing;
-    double temp, tolr;
+    real temp, tolr;
 
 /*     ********** */
 
