@@ -3,19 +3,21 @@
 	-lf2c -lm   (in that order)
 */
 
-#include <math.h>
 #include "cminpack.h"
+#include <math.h>
+#define real __cminpack_real__
 #define min(a,b) ((a) <= (b) ? (a) : (b))
 #define max(a,b) ((a) >= (b) ? (a) : (b))
 #define TRUE_ (1)
 #define FALSE_ (0)
 
-/* Subroutine */ int lmstr(minpack_funcderstr_mn fcn, void *p, int m, int n, double *x, 
-	double *fvec, double *fjac, int ldfjac, double ftol,
-	double xtol, double gtol, int maxfev, double *
-	diag, int mode, double factor, int nprint,
-	int *nfev, int *njev, int *ipvt, double *qtf, 
-	double *wa1, double *wa2, double *wa3, double *wa4)
+__cminpack_attr__
+int __cminpack_func__(lmstr)(__cminpack_decl_fcnderstr_mn__ void *p, int m, int n, real *x, 
+	real *fvec, real *fjac, int ldfjac, real ftol,
+	real xtol, real gtol, int maxfev, real *
+	diag, int mode, real factor, int nprint,
+	int *nfev, int *njev, int *ipvt, real *qtf, 
+	real *wa1, real *wa2, real *wa3, real *wa4)
 {
     /* Initialized data */
 
@@ -26,18 +28,18 @@
 #define p0001 1e-4
 
     /* System generated locals */
-    double d1, d2;
+    real d1, d2;
 
     /* Local variables */
     int i, j, l;
-    double par, sum;
+    real par, sum;
     int sing;
     int iter;
-    double temp, temp1, temp2;
+    real temp, temp1, temp2;
     int iflag;
-    double delta = 0.;
-    double ratio;
-    double fnorm, gnorm, pnorm, xnorm = 0., fnorm1, actred, dirder, 
+    real delta = 0.;
+    real ratio;
+    real fnorm, gnorm, pnorm, xnorm = 0., fnorm1, actred, dirder, 
 	    epsmch, prered;
     int info;
 
@@ -220,7 +222,7 @@
 
 /*     epsmch is the machine precision. */
 
-    epsmch = dpmpar(1);
+    epsmch = __cminpack_func__(dpmpar)(1);
 
     info = 0;
     iflag = 0;
@@ -244,12 +246,12 @@
 /*     evaluate the function at the starting point */
 /*     and calculate its norm. */
 
-    iflag = (*fcn)(p, m, n, x, fvec, wa3, 1);
+    iflag = fcnderstr_mn(p, m, n, x, fvec, wa3, 1);
     *nfev = 1;
     if (iflag < 0) {
 	goto TERMINATE;
     }
-    fnorm = enorm(m, fvec);
+    fnorm = __cminpack_func__(enorm)(m, fvec);
 
 /*     initialize levenberg-marquardt parameter and iteration counter. */
 
@@ -265,7 +267,7 @@
         if (nprint > 0) {
             iflag = 0;
             if ((iter - 1) % nprint == 0) {
-                iflag = (*fcn)(p, m, n, x, fvec, wa3, 0);
+                iflag = fcnderstr_mn(p, m, n, x, fvec, wa3, 0);
             }
             if (iflag < 0) {
                 goto TERMINATE;
@@ -285,11 +287,11 @@
         }
         iflag = 2;
         for (i = 0; i < m; ++i) {
-            if ((*fcn)(p, m, n, x, fvec, wa3, iflag) < 0) {
+            if (fcnderstr_mn(p, m, n, x, fvec, wa3, iflag) < 0) {
                 goto TERMINATE;
             }
             temp = fvec[i];
-            rwupdt(n, fjac, ldfjac, wa3, qtf, &temp,
+            __cminpack_func__(rwupdt)(n, fjac, ldfjac, wa3, qtf, &temp,
                    wa1, wa2);
             ++iflag;
         }
@@ -304,10 +306,10 @@
                 sing = TRUE_;
             }
             ipvt[j] = j+1;
-            wa2[j] = enorm(j+1, &fjac[j * ldfjac + 0]);
+            wa2[j] = __cminpack_func__(enorm)(j+1, &fjac[j * ldfjac + 0]);
         }
         if (sing) {
-            qrfac(n, n, fjac, ldfjac, TRUE_, ipvt, n,
+            __cminpack_func__(qrfac)(n, n, fjac, ldfjac, TRUE_, ipvt, n,
                   wa1, wa2, wa3);
             for (j = 0; j < n; ++j) {
                 if (fjac[j + j * ldfjac] != 0.) {
@@ -343,7 +345,7 @@
             for (j = 0; j < n; ++j) {
                 wa3[j] = diag[j] * x[j];
             }
-            xnorm = enorm(n, wa3);
+            xnorm = __cminpack_func__(enorm)(n, wa3);
             delta = factor * xnorm;
             if (delta == 0.) {
                 delta = factor;
@@ -393,7 +395,7 @@
 
 /*           determine the levenberg-marquardt parameter. */
 
-            lmpar(n, fjac, ldfjac, ipvt, diag, qtf, delta,
+            __cminpack_func__(lmpar)(n, fjac, ldfjac, ipvt, diag, qtf, delta,
                   &par, wa1, wa2, wa3, wa4);
 
 /*           store the direction p and x + p. calculate the norm of p. */
@@ -403,7 +405,7 @@
                 wa2[j] = x[j] + wa1[j];
                 wa3[j] = diag[j] * wa1[j];
             }
-            pnorm = enorm(n, wa3);
+            pnorm = __cminpack_func__(enorm)(n, wa3);
 
 /*           on the first iteration, adjust the initial step bound. */
 
@@ -413,12 +415,12 @@
 
 /*           evaluate the function at x + p and calculate its norm. */
 
-            iflag = (*fcn)(p, m, n, wa2, wa4, wa3, 1);
+            iflag = fcnderstr_mn(p, m, n, wa2, wa4, wa3, 1);
             ++(*nfev);
             if (iflag < 0) {
                 goto TERMINATE;
             }
-            fnorm1 = enorm(m, wa4);
+            fnorm1 = __cminpack_func__(enorm)(m, wa4);
 
 /*           compute the scaled actual reduction. */
 
@@ -440,7 +442,7 @@
                     wa3[i] += fjac[i + j * ldfjac] * temp;
                 }
             }
-            temp1 = enorm(n, wa3) / fnorm;
+            temp1 = __cminpack_func__(enorm)(n, wa3) / fnorm;
             temp2 = (sqrt(par) * pnorm) / fnorm;
             prered = temp1 * temp1 + temp2 * temp2 / p5;
             dirder = -(temp1 * temp1 + temp2 * temp2);
@@ -488,7 +490,7 @@
                 for (i = 0; i < m; ++i) {
                     fvec[i] = wa4[i];
                 }
-                xnorm = enorm(n, wa2);
+                xnorm = __cminpack_func__(enorm)(n, wa2);
                 fnorm = fnorm1;
                 ++iter;
             }
@@ -541,7 +543,7 @@ TERMINATE:
 	info = iflag;
     }
     if (nprint > 0) {
-	(*fcn)(p, m, n, x, fvec, wa3, 0);
+	fcnderstr_mn(p, m, n, x, fvec, wa3, 0);
     }
     return info;
 
